/******************************************************************************
*
* project name: TI-MineSweeper
* filename:     score.c
* initial date: 04/07/2000
* authors:      thomas.nussbaumer@gmx.net (coding)
*               marcos.lopez@gmx.net (game play/graphics/BETA testing)
* description:  highscore handling routines
*
* $Id: score.c,v 1.1 2002/09/27 10:10:29 tnussb Exp $
*
*******************************************************************************/

//---------------------------------------------
// length of playername array in highscore
// structure (don't change without increasing
// VERSION_FILEFORMAT)
//---------------------------------------------
#define PLAYER_NAME_LENGTH 20

//---------------------------------------------
// structure to hold a highscore (don't change
// without increasing VERSION_FILEFORMAT)
//---------------------------------------------
typedef struct {
   short  time;
   char   name[PLAYER_NAME_LENGTH];
} highscore_t;


//---------------------------------------------
// if highscore_t is changed, you must change
// ALSO this define to make sure that highscore
// files which contains entries in the old
// format are not used !!!!
//---------------------------------------------
#define VERSION_FILEFORMAT 1


//---------------------------------------------
// number of highscores
//---------------------------------------------
#define NR_HIGHSCORES 4


//---------------------------------------------
// default values of highscores for all modes
//---------------------------------------------
highscore_t defaults[NR_HIGHSCORES] = {
    {999,"       Programming "},
    {999,"  Thomas Nussbaumer"},
    {999,"   Gameplay and Gfx"},
    {999,"      Marcos Lopez "}
};


//---------------------------------------------
// active highscores are stored here !!
//---------------------------------------------
highscore_t scores[NR_HIGHSCORES] = {
    {999,"       Programming "},
    {999,"  Thomas Nussbaumer"},
    {999,"   Gameplay and Gfx"},
    {999,"      Marcos Lopez "}
};


//---------------------------------------------
// variable name to use for highscores
// a more or less confuse name is chosen to
// make relatively sure that it won't conflict
// with an existing variable on the calculator
//---------------------------------------------
#define HIGHSCORE_VARNAME "timinhsc"
#define HIGHSCORE_SYMNAME $(timinhsc)

/*===========================================================================*/
/* returns true if highscore variable is archived                            */
/*===========================================================================*/
short isArchivedOrLocked(void) {
    SYM_ENTRY* symentry;

    if (!(symentry = DerefSym(SymFind(HIGHSCORE_SYMNAME)))) return 0;

    return (symentry->flags.bits.archived || symentry->flags.bits.locked);
}

/*===========================================================================*/
/* write highscores to variable (type == STR)                                */
/*===========================================================================*/
void WriteHighScores() {
    short i,j;
    FILE* fp;

    //------------------------------------------------
    // if highscore variable is archived or locked
    // -> do nothing
    //------------------------------------------------
    if (isArchivedOrLocked()) return;

    if (!(fp = fopen(HIGHSCORE_VARNAME, "wb"))) return;

    fputc(0,fp);                  // write leading zero
    fputc(VERSION_FILEFORMAT,fp); // write file version number

    //--------------------------
    // write highscores contents
    //--------------------------
    for (j=0;j<NR_HIGHSCORES;j++) {
        fputc(scores[j].time & 0xff,fp);
        fputc((scores[j].time >> 8) & 0xff,fp);
        for (i=0;i<PLAYER_NAME_LENGTH;i++) fputc(scores[j].name[i],fp);
    }

    //---------------------------
    // write STR variable trailer
    //---------------------------
    fputc(0, fp);
    fputc(STR_TAG,fp);
    fclose(fp);
}


/*===========================================================================*/
/* load highscores from variable (type == STR)                               */
/*===========================================================================*/
void LoadHighScores(void) {
    short i,j;
    FILE* fp;

    //-------------------------------------------------------
    // if there isn't already a highscore file -> write a
    // default one
    //-------------------------------------------------------
    if (!(fp = fopen(HIGHSCORE_VARNAME,"rb"))) {
        memcpy(scores,defaults,sizeof(highscore_t)*4);
        WriteHighScores();
        return;
    }

    fgetc(fp); // skip leading zero

    //-------------------------------------------------------
    // check version number and write highscores if version
    // number is okay
    //-------------------------------------------------------
    if (fgetc(fp) == VERSION_FILEFORMAT) {
        for (j=0;j<NR_HIGHSCORES;j++) {
            scores[j].time =  fgetc(fp);
            scores[j].time += (fgetc(fp) <<8);
            for (i=0;i<PLAYER_NAME_LENGTH;i++) scores[j].name[i] = fgetc(fp);
        }
        fclose(fp);
        return;
    }

    //-------------------------------------------------------
    // we will only come here if the version number won't fit
    // -> close file and write a default one
    //-------------------------------------------------------
    fclose(fp);
    memcpy(scores,defaults,sizeof(highscore_t)*4);
    WriteHighScores();
}

//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: score.c,v $
// Revision 1.1  2002/09/27 10:10:29  tnussb
// initial check-in to CVS repository
//
//
